;==========================================================================
        Title  "Bike Advance"
;   http://www.transmic.net/
;       26 Mars 08      Version 4.41  for 16f628A     Thierry.P
;      16f628 Horloge interne 4MHz
;      cablage compatible cdi PCM5.3 sans quartz
;      Copyright  Thierry PERCIN
; -------------------------------------------------------------------------
; Adaptation of the program Ign.txt "Auto Advance" from A. Nixon
; Title  "Auto Advance" for 16c54
;       24 08 94      Version 1.0      A. Nixon
;==========================================================================

        ;
        ; Ign advance starts at 1000 engine RPM
        ;
        ; Ign full advance is at 8000 engine RPM
        ;
        ; dwell time is fixed at 1mS, =  10 X 100uS loops
        ;
        ; After 1000 RPM, the ignition advance is calculated in 131 steps.
        ; This continues up until 8000 RPM is reached.
        ; 
        ;
        ; At a engine speed of 1200 RPM;
        ;
        ;        1200 / 60 = 20 revs per second
        ;          1 / 20 = 50mS per rev
        ;           1 rev = 360 degrees
        ;
        ;        36 degrees at 1200 RPM = (36 / 360) X 50 = 5mS
        ;
        ; Therefore maximum retard at 1200 RPM = 5mS after the firing
        ; point is reached. This figure remains constant down to 0 RPM.
        ;
        ; At a engine speed of 8000 RPM, maximum advance will have been reached,
        ; and ignition is immediately after the firing point is reached.
        ;
        ; The intermediate values will be looked up in an advance table map
        ; and will correspond to engine speed from 1000 RPM to 8000 RPM.
        ;
        ; As the engine RPM increases above 1000, the retard value becomes
        ; progressively lower until 8000 RPM is reached, whereby the 
        ; retard value will be 0.
        ; Under 1000 RPM the retard value is fixed at 5ms (maxret)
        ;
        ;
        ; Map holds retard values in 100uS loops for every loop change
        ; in rpm rate.
        ;
        ;
        ;
;==========================================================================
;
;       PIC 16F628A v5.4 wiring
;
;==========================================================================

	;                  
	;  LEDrpmLimit  <------- RA2   1+--O--+18  RA1
	;  ReverseOutput<------- RA3   2|     |17  RA0
	;                        RA4   3|     |16  Osc
	;  +5V-------RRR-------> MCLR  4|     |15  Osc
	;  Ground--------------- Vss   5|     |14  Vcc ------- +5Vdc
	;                        RB0   6|     |13  RB7 <------ switch (map selection)
	;                        RB1   7|     |12  RB6 ------> pickupLED
	;  Thyristor    <------- RB2   8|     |11  RB5
	;                        RB3   9+-----+10  RB4 <------ pickup36

;==========================================================================
;
;       Versions
;
;==========================================================================

        ; Version 2.3: add a reverse output to coil, detect rotor __-__
        ; Version 2.6: modif calculation, detect rotor --_--
        ; Version 2.8: modif register
        ; Version 3.0: correction at low RPM, or big retard.
        ; Version 3.1: deboun=1ms, initial delay=18ms, detect rotor high __-__
        ; Version 3.2: suppression extra pluses at low RPM
        ; Version 4.0: simplification, 4MHz cristal.trigger on decreasing edge
        ; Version 4.1: trigger on rising edge
        ; Version 4.2: map move at the end of the soft. max adv=0ms
        ; Version 4.30: 2 maps selectables
        ; Version 4.40: 8MHz cristal.A different PCB
        ; Version 4.41: for 16F628A internal clock 4MHz
        ;
        ;

;==========================================================================
;
;       Directives
;
;==========================================================================
; mplab settings

;	ERRORLEVEL -224		; suppress annoying message because of option/tris
	ERRORLEVEL -302		; suppress message because of bank select in setup ports
         LIST   P=PIC16F628A    	; processor type
;==============================================================================
;
        #include "P16F628A.INC"  ; Include header file

        __config  _INTRC_OSC_NOCLKOUT & _LVP_OFF & _WDT_OFF & _PWRTE_ON & _BODEN_ON & _MCLRE_OFF
;==========================================================================
;
;       RAM Definition
;
;==========================================================================

        CBLOCK 0x20             ; Declare variable addresses starting at 0x20
          rpmhi,rpmlo,dwell,rtdset,math,coilflg,spkflg,rotflg,retard
        ENDC

; rpmhi       RPM counter register high
; rpmlo       RPM counter register low
; dwell       dwell time counter
; rtdset      retard value holding register
; math        calculation register
; coilflg     coil status
; spkflg      flag after one spark
; rotflg      rotor sense flag
; retard      new retard value

;==========================================================================
;
;       Define
;
;==========================================================================

#DEFINE table	PORTB,7     	; [pin13]= map selecting
#DEFINE rotor	PORTB,4     	; [pin10]= sensor input
#DEFINE coil	PORTB,2   	; [pin8]= output to coil
#DEFINE revcoil	PORTA,3		; [pin2]= reverse output to coil
#DEFINE rpmmax	PORTA,2    	; [pin1]= Min and Max RPM led
#DEFINE led	PORTB,6       	; [pin12]= pickup led

;==========================================================================
;
;       Main Program
;
;==========================================================================


        org 0			; start adress 0
start	movlw B'00000111'	; set comparators off
        movwf CMCON		; PORT A as I/O
        bsf    STATUS,RP0  	; go to bank 1
        movlw  H'00'       	; set all portA..
        movwf  TRISA		; ..as outputs
        movlw  B'10010000'      ; bit 0,1,2,3,5,6 = o/p's, 4,7 = i/p's
        movwf  TRISB		; set portB
        ;
        movlw b'11000001'	; no pull-up,rising edge int,internal timer,1:4
        movwf OPTION_REG
        bcf  STATUS,RP0    	; come back into bank 0
        ;
        clrw               	; clear all variables
        clrf PORTA         	; clear variables
        clrf PORTB
        clrf math
        clrf rpmlo
        clrf coilflg
        movlw 0xff         	; rpmhi = FF initially
        movwf rpmhi
        movwf spkflg		; no spark until pickup is high
        movwf rotflg		; and no spark loop if pickup stay up
        movwf retard		; retard value = 25ms initially
        ;
        ;
        ;       100uS loop time
        ;       4MHz clock / 4 = 1MHz instruction cycle
        ;       cycle time = 1/1MHz  =  100uS
        ;       100uS/100uS  =  100 cycles
        ;       prescaler set to divide by 4
        ;       preset TMR0 = 231, when = 0  =  100uS
        ;       231 to 255 = 25 x 4(prescaler) = 100
        ;
        movlw .231
        movwf TMR0
        ;
        ;
mnloop  movf TMR0,W
        btfss STATUS,Z		; is TMR0=0 ?
        goto mnloop     	; no, main delay loop
        ;
        movlw .231		; yes, re-preset TMR0 231
        movwf TMR0
        ;
        incf rpmlo,F    	; increment rpmlo
        btfsc STATUS,Z  	; is rpmlo = 0 ?
        incf rpmhi,F    	; yes, increment rpmhi
        ;
        ;       if rpm count > 1400h, 5120 x 100us=0.5 Sec, then it is assumed that the
        ;       motor has stopped. Reinitialize system.
        movlw 0x14
        subwf rpmhi,W
        btfsc STATUS,Z     	; rpmhi - 14 = 0 ?
        goto start         	; yes
        ;
        
rotchk  btfss rotor		; + is rotor high or low?
        ;btfsc rotor		; (- detect pickup low)
        goto rnowlw             ; low
        ;
        movf rotflg,W   	; high, is it flagged high also? rotflg=FF?
        btfss STATUS,Z
        goto coilck     	; yes
        ;
        movlw 0xff 		; no
        movwf rotflg     	; flag rotor as up
        bsf led			; turn ignition pickup led on
        clrf spkflg	      	; reset flag for one spark
        goto rpmcalc		; calculate RPM
        ;

rnowlw  movf rotflg,W   	; is it flagged low already ? rotflg=0?
        btfsc STATUS,Z
        goto coilck 	    	; yes, wasn't flagged high
        ;
        
dolow   bcf led		   	; no, turn ignition pickup led off
        clrf rotflg     	; flag rotor as down
        goto mnloop
        ;

decret	decf retard,F		; no, decrement retard then wait
	goto mnloop
	;
	                
coilck  movf coilflg,W	   	; is coil flagged high? coilflg=FF?
        btfss STATUS,Z
        goto upcoil             ; yes, then decrement dwell count
        movf retard,W		; is retard = 0 ?
        btfss STATUS,Z
        goto decret		; no, then decrement retard
        ;
        movf spkflg,W	   	; yes, a spark have been done? spkflg=FF?
        btfss STATUS,Z
        goto mnloop             ; yes, no more spark
        ;
        bsf coil		; no, go on for the first spark
        bcf revcoil
        movlw 0xff      	; set flag for high coil
        movwf coilflg
        movwf spkflg		; set flag for one spark

        ;   ****************** YOU CAN MODIFY THE DWELL VALUE BELLOW ********
        movlw .20		; 20 X 100uS  = 2mS
        movwf dwell		; set dwell time
        ;
        goto mnloop
        ;

upcoil  ;       		if dwell # 0 then dwell - 1
        ;			if dwell now = 0, turn on coil
        ; 
        decfsz dwell,F  	; dwell = dwell - 1
        goto mnloop     	; dwell <> 0, wait
        bcf coil		; dwell = 0, turn coil off
        bsf revcoil	  	; dwell = 0, turn revcoil on
        clrf coilflg	      	; set flag for low coil
        goto mnloop
        ;

rpmcalc ;       engine RPM <= 1000 at maximum retard, = 60ms
        ;
        ;       60mS = 600 X 100us loops, = 258Ch
        ;       ( 1000RPM for a 1cyl - 4stroke with one spark loose )
        ;       ( 1000RPM for a 1cyl - 2stroke )
        ;       (  500RPM for a 4cyl - 4stroke = 16Hz )
        ;
        ;       engine RPM >= 8000 at maximum advance, = 7.5mS
        ;       7.5mS = 75 X 100uS loops, = 4Bh
        ;       ( 8000RPM for a 1cyl - 4stroke with one spark loose )
        ;       ( 8000RPM for a 1cyl - 2stroke )
        ;       (  4000RPM for a 4cyl - 4stroke = 133Hz )
        ;
        ;       this routine determines whether the engine RPM
        ;       value is below 75 loop counts - max advance, or above
        ;       600 loop counts - max retard, or some where in between.
        ;       
        ;       
        ; high RPM:     	; > 8000RPM
        ;
        movf rpmhi,W
        btfss STATUS,Z  	; is rpmhi = 0?
        goto bigfv      	; no
        ;
        movf rpmlo,W
        xorlw 0x4B	      	; yes, is rpmlo = 4Bh ?
        btfsc STATUS,Z
        goto maxadv     	; yes, do max advance
        ;
        movlw 0x4B
        subwf rpmlo,W   	; no, is rpmlo < 4Bh ?
        btfss STATUS,C
        goto maxadv     	; yes, do max advance
        ;
        ; medium RPM:
        goto caladv     	; no, calculate new advance
        ;
        ;
        ; low RPM:      	; < 1000RPM
bigfv   movf rpmhi,W
        xorlw .2        	; is rpmhi = 2 ?
        btfsc STATUS,Z
        goto resfv      	; yes, see if rpmlo = 58h
        ;
        movlw .2
        subwf rpmhi,W   	; no, is high byte > 2 ?
        btfsc STATUS,C
        goto maxret     	; yes, do max retard
        ;
        goto caladv     	; no, calculate new advance
        ;
resfv   movf rpmlo,W
        xorlw 0x58      	; rpmhi = 2, does rpmlo = 58h ?
        btfsc STATUS,Z
        goto maxret     	; yes, do max retard
        ;
        movlw 0x58
        subwf rpmlo,W   	; no, is rpmlo > 58h ?
        btfsc STATUS,C
        goto maxret     	; yes, do max retard
        ;
        ;
        ; the formula to get the data stored in the map is as follows
        ;
        ;                 rpmhi/lo count  -  74
        ;    131   -    ---------------------------
        ;                           4
        ;
caladv  movlw d'74'    		; rpmhi/lo - 74
        subwf rpmlo,F		; rpmlo = rpmlo - 74
        btfss STATUS,C
        decf rpmhi,F
        ;
        ; divide result by 4
        ;
        bcf STATUS,C
        rrf rpmhi,F
        rrf rpmlo,F     	; / 2
        ;
        bcf STATUS,C
        rrf rpmhi,F
        rrf rpmlo,F     	; / 4
        ;
        movlw .131      	; 131 entries in map list
        movwf math
        ;
        movf rpmlo,W
        subwf math,W    	; W = 131 - result
        ;
        bcf PCLATH,0		; default map is at h'200'
        bsf PCLATH,1
        btfss table		; is table select low?
        bsf PCLATH,0		; yes,then select table N2 at h'300'
        ;
        call map		; read map
        movwf rtdset    	; come back with new retard value
        bcf rpmmax	 	; turn debug led off
        ;
rpmset  clrf rpmhi		; clear RPM counters
        clrf rpmlo
        movf rtdset,W		; transfert advance value
        movwf retard
        goto mnloop
        ;
maxret  movlw 31h		; < 1000RPM
        movwf rtdset     	; retard value = 31h (~4.9ms)  16hz
        bsf rpmmax 		; turn led on
        goto rpmset
        ;
maxadv  clrf rtdset      	; > 8000RPM retard value = 0ms	124hz
        bsf rpmmax	 	; turn led on
        goto rpmset
        ;
map     org h'200'		; store map1 at 200h
        addwf PCL,1		; add W + PCL
        ;  *********** INSERT YOUR OWN VALUES HERE *******************
        retlw 31h     ;4,9ms   1000rpm    
        retlw 30h     ;4,8ms   1016rpm 
        retlw 2Fh     ;4,7ms   1023rpm 
        retlw 2Fh     ;4,7ms   1030rpm 
        retlw 2Eh     ;4,6ms   1038rpm
        retlw 2Dh     ;4,5ms   1045rpm 
        retlw 2Dh     ;4,5ms   1052rpm 
        retlw 2Ch     ;4,4ms   1060rpm 
        retlw 2Bh     ;4,3ms   1067rpm 
        retlw 2Ah     ;4,2ms   1075rpm 
        retlw 2Ah     ;4,2ms   1083rpm 
        retlw 29h     ;4,1ms   1090rpm 
        retlw 28h     ;4,0ms   1098rpm
        retlw 28h     ;4,0ms   1107rpm 
        retlw 27h     ;3,9ms   1115rpm 
        retlw 26h     ;3,8ms   1123rpm 
        retlw 26h     ;3,8ms   1132rpm 
        retlw 25h     ;3,7ms   1140rpm 
        retlw 24h     ;3,6ms   1149rpm 
        retlw 24h     ;3,6ms   1158rpm 
        retlw 23h     ;3,5ms   1167rpm 
        retlw 22h     ;3,4ms   1176rpm 
        retlw 22h     ;3,4ms   1185rpm 
        retlw 21h     ;3,3ms   1195rpm 
        retlw 20h     ;3,2ms   1204rpm 
        retlw 20h     ;3,2ms   1214rpm 
        retlw 1Fh     ;3,1ms   1224rpm 
        retlw 1Eh     ;3,0ms   1234rpm 
        retlw 1Eh     ;3,0ms   1244rpm 
        retlw 1Dh     ;2,9ms   1255rpm 
        retlw 1Dh     ;2,9ms   1265rpm 
        retlw 1Ch     ;2,8ms   1276rpm 
        retlw 1Ch     ;2,8ms   1287rpm 
        retlw 1Bh     ;2,7ms   1298rpm 
        retlw 1Ah     ;2,6ms   1310rpm 
        retlw 1Ah     ;2,6ms   1321rpm 
        retlw 19h     ;2,5ms   1333rpm 
        retlw 19h     ;2,5ms   1345rpm 
        retlw 18h     ;2,4ms   1357rpm 
        retlw 18h     ;2,4ms   1369rpm 
        retlw 17h     ;2,3ms   1382rpm 
        retlw 17h     ;2,3ms   1395rpm 
        retlw 16h     ;2,2ms   1408rpm 
        retlw 16h     ;2,2ms   1421rpm 
        retlw 15h     ;2,1ms   1435rpm 
        retlw 14h     ;2,0ms   1449rpm 
        retlw 14h     ;2,0ms   1463rpm 
        retlw 14h     ;2,0ms   1477rpm 
        retlw 13h     ;1,9ms   1492rpm 
        retlw 13h     ;1,9ms   1507rpm 
        retlw 12h     ;1,8ms   1522rpm 
        retlw 12h     ;1,8ms   1538rpm 
        retlw 11h     ;1,7ms   1554rpm 
        retlw 11h     ;1,7ms   1570rpm 
        retlw 10h     ;1,6ms   1587rpm 
        retlw 10h     ;1,6ms   1604rpm 
        retlw 0Fh     ;1,5ms   1621rpm 
        retlw 0Fh     ;1,5ms   1639rpm 
        retlw 0Eh     ;1,4ms   1657rpm 
        retlw 0Eh     ;1,4ms   1675rpm 
        retlw 0Eh     ;1,4ms   1694rpm 
        retlw 0Dh     ;1,3ms   1714rpm 
        retlw 0Dh     ;1,3ms   1734rpm 
        retlw 0Ch     ;1,2ms   1754rpm 
        retlw 0Ch     ;1,2ms   1775rpm 
        retlw 0Ch     ;1,2ms   1796rpm 
        retlw 0Bh     ;1,1ms   1818rpm 
        retlw 0Bh     ;1,1ms   1840rpm 
        retlw 0Ah     ;1,0ms   1863rpm 
        retlw 0Ah     ;1,0ms   1886rpm 
        retlw 0Ah     ;1,0ms   1910rpm 
        retlw 9h     ;1,0ms    1935rpm 
        retlw 9h     ;1,0ms    1960rpm 
        retlw 9h     ;0,9ms   1986rpm 
        retlw 8h     ;0,9ms   2013rpm 
        retlw 8h     ;0,9ms   2040rpm 
        retlw 8h     ;0,8ms   2068rpm 
        retlw 7h     ;0,8ms   2097rpm 
        retlw 7h     ;0,8ms   2127rpm 
        retlw 7h     ;0,7ms   2158rpm 
        retlw 6h     ;0,7ms   2189rpm 
        retlw 6h     ;0,7ms   2222rpm 
        retlw 6h     ;0,6ms   2255rpm 
        retlw 6h     ;0,6ms   2290rpm 
        retlw 5h     ;0,6ms   2325rpm 
        retlw 5h     ;0,6ms   2362rpm 
        retlw 5h     ;0,5ms   2400rpm 
        retlw 4h     ;0,5ms   2439rpm 
        retlw 4h     ;0,5ms   2479rpm 
        retlw 4h     ;0,4ms   2521rpm 
        retlw 4h     ;0,4ms   2564rpm 
        retlw 3h     ;0,4ms   2608rpm 
        retlw 3h     ;0,4ms   2654rpm 
        retlw 3h     ;0,4ms   2702rpm 
        retlw 3h     ;0,3ms   2752rpm 
        retlw 3h     ;0,3ms   2803rpm 
        retlw 2h     ;0,3ms   2857rpm 
        retlw 2h     ;0,3ms   2912rpm 
        retlw 2h     ;0,2ms   2970rpm 
        retlw 2h     ;0,2ms   3030rpm 
        retlw 2h     ;0,2ms   3092rpm 
        retlw 1h     ;0,2ms   3157rpm 
        retlw 1h     ;0,2ms   3225rpm 
        retlw 1h     ;0,2ms   3296rpm 
        retlw 1h     ;0,1ms   3370rpm 
        retlw 1h     ;0,1ms   3448rpm 
        retlw 1h     ;0,1ms   3529rpm 
        retlw 1h     ;0,1ms   3614rpm 
        retlw 1h     ;0,1ms   3703rpm 
        retlw 1h     ;0,1ms   3797rpm 
        retlw 1h     ;0,1ms   3896rpm 
        retlw 1h     ;      4000rpm 
        retlw 1h     ;      4109rpm 
        retlw 1h     ;      4225rpm 
        retlw 1h     ;      4347rpm 
        retlw 1h     ;      4477rpm 
        retlw 1h     ;      4615rpm 
        retlw 1h     ;      4761rpm 
        retlw 1h     ;      4918rpm 
        retlw 0h     ;      5084rpm 
        retlw 0h     ;      5263rpm 
        retlw 0h     ;      5454rpm 
        retlw 0h     ;      5660rpm 
        retlw 0h     ;      5882rpm 
        retlw 0h     ;      6122rpm 
        retlw 0h     ;      6382rpm 
        retlw 0h     ;      6666rpm 
        retlw 0h     ;      6976rpm 
        retlw 0h     ;      7317rpm 
        retlw 0h     ;      7692rpm 
        retlw 0h     ;
        ;  ********* END OF YOUR OWN VALUES ******************
        retlw 31h     ; in case of overlap
        ; line565
        ;
        
map2    org h'300'		; store second map at 300h
        addwf PCL,1		; add W + PCL
        ;  *********** INSERT YOUR OTHERS VALUES HERE ***************
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        retlw 0h     ;0ms
        ;  ********* END OF YOUR OWN VALUES ******************
        retlw 31h     ; in case of overlap
        ; line704
        ;
        ;       reset vector
        ;
        ;
        org 400h		; if the program is loose,
        ;
        ;
        ;
        goto start		; It goes back home.
        ;
        ;
        ;
        end
